<?php
/**
 * Ce fichier contient les fonctions de service de Mashup Factory pour les autorisations, les feeds et les datasets.
 *
 * Chaque fonction, soit aiguille, si elle existe, vers une fonction "homonyme" propre au plugin appelant
 * ou à un feed, soit déroule sa propre implémentation.
 * Ainsi, les plugins externes peuvent, si elle leur convient, utiliser l'implémentation proposée par Mashup Factory
 * en codant un minimum de fonctions.
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_EXTRAIRE_MULTI')) {
	/**
	 * Restaure cette constante à partir de SPIP 4.2.
	 */
	define('_EXTRAIRE_MULTI', '@<multi>(.*?)</multi>@sS');
}
if (!defined('_EXTRAIRE_IDIOME')) {
	/**
	 * Extrait les composants d'un idiome de langue utilisable dans un YAML
	 * '@<:(?:([a-z0-9_]+):)?([a-z0-9_]+):>@isS'.
	 */
	define('_EXTRAIRE_IDIOME', ',<:(?:([a-z0-9_]+):)?([a-z0-9_]*):/?>,iS');
}

// -----------------------------------------------------------------------
// ----------------------- AUTORISATION DE BASE --------------------------
// -----------------------------------------------------------------------

/**
 * Renvoie l'autorisation minimale d'un plugin utilisateur de Mashup Factory.
 *
 * Le plugin Mashup Factory autorise, par défaut, les administrateurs complets.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string         $plugin Préfixe du plugin utilisateur.
 * @param null|array|int $qui    L'initiateur de l'action:
 *                               - si null on prend alors visiteur_session
 *                               - un id_auteur (on regarde dans la base)
 *                               - un tableau auteur complet, y compris [restreint]
 *
 * @return bool `true`si l'auteur est autorisée à l'utilisation de base du plugin Mashup Factory, `false` sinon.
 */
function ezmashup_plugin_autoriser(string $plugin, $qui) : bool {
	// Si le plugin appelant complète la liste des catégories ou la modifie, il doit fournir un service associé.
	if ($autoriser = ezmashup_chercher_service($plugin, 'plugin_autoriser')) {
		// On passe le plugin appelant à la fonction car cela permet ainsi de mutualiser les services de stockage.
		$autorise = $autoriser($plugin, $qui);
	} else {
		include_spip('inc/autoriser');
		$autorise = autoriser('defaut');
	}

	return $autorise;
}

// -----------------------------------------------------------------------
// ------------------------------ FEEDS ----------------------------------
// -----------------------------------------------------------------------

/**
 * Renvoie la liste des catégories et leur description.
 *
 * Le plugin Mashup Factory fournit une liste limitée par défaut.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 *
 * @return array Liste des catégories et de leur description au format [id] = tableau de description avec le nom (label),
 *               la description et l'icone.
 */
function ezmashup_feed_lister_categories(string $plugin) : array {
	// Initialisation des catégories par défaut
	$categories = [
		'default' => [
			'name'        => '<:ezmashup:label_feed_category_default:>',
			'description' => '<:ezmashup:description_feed_category_default:>',
			'icon'        => 'ezmashup_category-24.svg'
		],
	];

	// Si le plugin appelant complète la liste des catégories ou la modifie, il doit fournir un service associé.
	if ($lister = ezmashup_chercher_service($plugin, 'feed_lister_categories')) {
		// On passe le plugin appelant à la fonction car cela permet ainsi de mutualiser les services de stockage.
		$categories = $lister($plugin, $categories);
	}

	return $categories;
}

/**
 * Renvoie la configuration par défaut du dossier relatif où trouver les feeds.
 * Cette information est utilisée a minima au chargement des feeds disponibles.
 *
 * Le service de Mashup Factory considère que par défaut le dossier relatif des types de noisette est `feeds/`.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 *
 * @return string Chemin relatif du dossier où chercher les feeds.
 */
function ezmashup_feed_initialiser_dossier(string $plugin) : string {
	// Si le plugin utilisateur permet la configuration du dossier des feeds ou ne suit pas la configuration
	// de Mashup Factory, il doit proposer un service pour fournir cette valeur.
	if ($configurer = ezmashup_chercher_service($plugin, 'feed_initialiser_dossier')) {
		$dossier = $configurer($plugin);
	} else {
		// Le service ne propose pas de fonction propre, on utilise celle de Mashup Factory.
		$dossier = 'feeds/';
	}

	return $dossier;
}

/**
 * Renvoie la liste des catégories et leur description.
 *
 * Le plugin Mashup Factory fournit une liste limitée par défaut.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 *
 * @return string Identifiant de la catégorie par défaut.
 */
function ezmashup_feed_initialiser_categorie_defaut(string $plugin) : string {
	// Si le plugin utilisateur permet la configuration du défaut des catégories ou ne suit pas la configuration
	// de Mashup Factory, il doit proposer un service pour fournir cette valeur.
	if ($configurer = ezmashup_chercher_service($plugin, 'feed_initialiser_categorie_defaut')) {
		$categorie = $configurer($plugin);
	} else {
		// Initialisation des catégories par défaut de Mashup Factory
		$categorie = 'default';
	}

	return $categorie;
}

/**
 * Finalise l'exécution d'un feed uniquement si celle-ci s'est correctement déroulée.
 *
 * Le plugin Mashup Factory supprime les caches ezREST des feeds de façon à s'assurer que toute requête les concernant
 * remonte les informations les plus à jour et cohérentes avec le contenu des feeds.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses cache_reponse_vider()
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 * @param array  $feed   Configuration du feed
 *
 * @return void
 */
function ezmashup_feed_completer_execution(string $plugin, array $feed) : void {
	// Suppression des caches ezREST des feeds.
	include_spip('inc/ezrest_cache');
	cache_reponse_vider('ezmashup', '', 'feeds');

	// Si le plugin utilisateur, veut compléter le traitement par défaut de Mashup Factory,
	// il doit proposer un service idoine.
	if ($completer = ezmashup_chercher_service($plugin, 'feed_completer_execution')) {
		$completer($feed);
	}
}

// -----------------------------------------------------------------------
// ------------------------- TARGET DATASETS -----------------------------
// -----------------------------------------------------------------------

/**
 * Renvoie la configuration par défaut du dossier relatif où stocker les datasets cible de type fichier.
 *
 * Le service de Mashup Factory considère que par défaut le dossier relatif des types de noisette est `targets/`
 * et est inclus dans le dossier de base _DIR_ETC/ezmashup/.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 *
 * @return string Chemin relatif du dossier où chercher les feeds.
 */
function ezmashup_target_initialiser_dossier(string $plugin) : string {
	// Si le plugin utilisateur permet la configuration du dossier des feeds ou ne suit pas la configuration
	// de Mashup Factory, il doit proposer un service pour fournir cette valeur.
	if ($configurer = ezmashup_chercher_service($plugin, 'target_initialiser_dossier')) {
		$dossier = $configurer($plugin);
	} else {
		// Le service ne propose pas de fonction propre, on utilise celle de Mashup Factory.
		$dossier = 'targets/';
	}

	return $dossier;
}

/**
 * Complète éventuellement la consigne de peuplement d'un dataset cible.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin          Préfixe du plugin utilisateur.
 * @param array  $consigne        Consigne initialisé par défaut par Mashup Factory
 * @param array  $enregistrements Liste des enregistrement du dataset cible
 * @param array  $feed            Configuration du feed
 *
 * @return array Consigne éventuellement mise à jour.
 */
function ezmashup_target_completer_consigne(string $plugin, array $consigne, array $enregistrements, array $feed) : array {
	// Si besoin on appelle une fonction spécifique au feed pour compléter la consigne du peuplement du dataset
	$options = [
		'prefixe' => $feed['feed_id'],
		'include' => $feed['include']
	];
	if ($consigner = ezmashup_chercher_service($plugin, 'target_completer_consigne', $options)) {
		$consigne = $consigner($consigne, $enregistrements, $feed);
	}

	return $consigne;
}

/**
 * Complète un record déjà rempli avec les valeurs par défaut et les champs basiques et extra :
 * - construction d'un champ label multilangues
 * - traitements spécifiques au feed
 * - suppression des champs non utilisés dans le stockage de la cible.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin         Préfixe du plugin utilisateur.
 * @param array  $enregistrement Enregistrement du dataset cible déjà rempli
 * @param array  $feed           Configuration du feed
 *
 * @return array Enregistrement mis à jour.
 */
function ezmashup_record_completer(string $plugin, array $enregistrement, array $feed) : array {
	// Si la table possède un nom multilangue, on le calcule à partir des champs du type label_xx
	// où xx est le code de langue utilisé par SPIP (toujours un alpha2).
	// - mode pre-complétion
	if ($feed['mapping']['has_label_field'] === 'pre') {
		$enregistrement = ezmashup_record_compiler_label($enregistrement);
	}

	// Si besoin on appelle une fonction spécifique au feed pour chaque enregistrement afin de le compléter.
	$options = [
		'prefixe' => $feed['feed_id'],
		'include' => $feed['include']
	];
	if ($completer = ezmashup_chercher_service($plugin, 'record_completer', $options)) {
		$enregistrement = $completer($enregistrement, $feed);
	}

	// - mode post-complétion
	if ($feed['mapping']['has_label_field'] === 'post') {
		$enregistrement = ezmashup_record_compiler_label($enregistrement);
	}

	// Maintenant que l'enregistrement est entièrement complété on peut supprimer les champs
	// qui ne sont pas insérés dans la base (unused_fields)
	if (!empty($feed['mapping']['unused_fields'])) {
		foreach ($feed['mapping']['unused_fields'] as $_champ) {
			unset($enregistrement[$_champ]);
		}
	}

	return $enregistrement;
}

/**
 * Fusionne un record déjà rempli avec le record existant de même clé primaire dans la liste des records.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin                  Préfixe du plugin utilisateur.
 * @param array  $enregistrement_en_cours Enregistrement du dataset cible en cours de constitution
 * @param array  $enregistrement_en_liste Enregistrement du dataset cible déjà dans la liste
 * @param array  $feed                    Configuration du feed
 *
 * @return array Item mis à jour
 */
function ezmashup_record_fusionner(string $plugin, array $enregistrement_en_cours, array $enregistrement_en_liste, array $feed) : array {
	$options = [
		'prefixe' => $feed['feed_id'],
		'include' => $feed['include']
	];
	if ($fusionner = ezmashup_chercher_service($plugin, 'record_fusionner', $options)) {
		$enregistrement = $fusionner($enregistrement_en_cours, $enregistrement_en_liste, $feed);
	} else {
		// Il n'existe pas de fonction de fusion, c'est donc une erreur (clé en doublon) : on renvoie un enregistrement
		// vide pour le signaler.
		$enregistrement = [];
	}

	return $enregistrement;
}

/**
 * Complète un texte sous forme de balise multi avec une liste de traductions.
 * Les nouvelles traductions, si elles sont non vides, sont soit ajoutées, soit écrasent la traduction courante.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @param string $multi_in    Chaine au format `<multi>`.
 * @param array  $traductions Tableau des traductions à ajouter (format [langue] = texte).
 *
 * @return string Chaine complétée au format `<multi>`.
 */
function ezmashup_record_completer_traduction(string $multi_in, array $traductions): string {
	// On initialise la sortie avec le texte multi fourni en entrée.
	$multi_out = $multi_in;

	// On décompose le texte multi d'entrée en tableau de traductions
	if ($traductions) {
		$traductions_out = [];
		if (preg_match(_EXTRAIRE_MULTI, $multi_in, $match)) {
			$contenu_multi = trim($match[1]);
			if ($contenu_multi) {
				if (
					include_spip('inc/filtres')
					and function_exists('extraire_trads')
				) {
					// Filtre extraire_trads avant spip 4.2
					$traductions_out = extraire_trads($contenu_multi);
				} else {
					// Depuis spip 4.2 il faut passer par le collecteur Multis
					// -- on crée un objet collecteur multis
					include_spip('src/Texte/Collecteur/Multis');
					$collecteurMultis = new Spip\Texte\Collecteur\Multis();
					// -- on collecte les 2 multi en reconstituant la balise et on extrait les traductions
					//    car on est sur que le contenu est non vide.
					$multis = $collecteurMultis->collecter($multi_in);
					$traductions_out = $multis[0]['trads'];
				}
			}
		} else {
			// La chaine n'est pas en multi, on considère que c'est une traduction par défaut
			$traductions_out[''] = $multi_in;
		}

		// On boucle sur les nouvelles traductions à ajouter ou à mettre à jour
		foreach ($traductions as $_langue => $_traduction) {
			// Si la traduction d'une langue est vide on ne l'ajoute ni ne la modifie
			if ($_traduction) {
				$traductions_out[$_langue] = $_traduction;
			}
		}

		// On reconstitue le texte multi
		$multi_out = '';
		foreach ($traductions_out as $_langue => $_traduction) {
			if ($_traduction) {
				$multi_out .= ($_langue ? '[' . $_langue . ']' : '') . trim($_traduction);
			}
		}
		$multi_out = "<multi>{$multi_out}</multi>";
	}

	return $multi_out;
}

/**
 * Calcule un label multilangue à partir d'un enregistrement possédant des champs nommés `label_<code_de_langue>`
 * ou les champs `label` et `language`.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @internal
 *
 * @param array $enregistrement Enregistrement du dataset cible.
 *
 * @return array Enregistrement mis à jour avec le champ label même si celui-ci est vide.
 */
function ezmashup_record_compiler_label(array $enregistrement) {
	// On détermine la méthode de calcul suivant les champs existant dans l'enregistrement
	$label = '';
	if (isset($enregistrement['language'], $enregistrement['label'])) {
		$label = "[{$enregistrement['language']}]" . $enregistrement['label'];
	} else {
		// Constituer un champ label à partir des champs label_xx de l'enregistrement si ils existent
		foreach ($enregistrement as $_champ => $_valeur) {
			if ($_valeur
			and (substr($_champ, 0, 6) == 'label_')
			and ($langue = str_replace('label_', '', $_champ))) {
				$label .= "[{$langue}]{$_valeur}";
			}
		}
	}

	// On finalise le multi
	$enregistrement['label'] = $label
		? "<multi>{$label}</multi>"
		: '';

	return $enregistrement;
}

/**
 * Finalise la liste des records constituées à partir de la source.
 *
 * Par défaut, Mashup Factory calcule, si besoin, le champ profondeur pour les hiérarchies. Sinon, le plugin appelle
 * un traitement spécifique au feed, si nécessaire.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin          Préfixe du plugin utilisateur.
 * @param array  $enregistrements Liste des enregistrement du dataset cible
 * @param array  $feed            Configuration du feed
 *
 * @return array Enregistrement mis à jour
 */
function ezmashup_record_list_completer(string $plugin, array $enregistrements, array $feed) : array {
	// Si la configuration intègre un calcul de profondeur (pour les arborescences de territoires par exemple)
	// on appelle la fonction qui rajoute ce champ pour tous les enregistrements.
	if (!empty($feed['mapping']['depth_fields'])) {
		// Extraction de la configuration du champ désignant le parent
		$champ_parent = $feed['mapping']['depth_fields']['parent_field'];

		// On indexe la liste des enregistrements par le code unique de l'élément
		$enregistrements = array_column($enregistrements, null, $feed['mapping']['depth_fields']['parent_value']);

		foreach ($enregistrements as $_cle => $_enregistrement) {
			$profondeur = $feed['mapping']['depth_fields']['base_value'];
			if ($code_parent = $_enregistrement[$champ_parent]) {
				// Un parent existe, on remonte la parenté pour définir la profondeur
				while ($code_parent) {
					if (isset($enregistrements[$code_parent])) {
						// Le parent est du même type, on continue à remonter la parenté
						++$profondeur;
						$code_parent = $enregistrements[$code_parent][$champ_parent];
					} else {
						// Le parent n'est pas du même type, cela revient à être au top du type
						$code_parent = '';
					}
				}
			}
			$enregistrements[$_cle][$feed['mapping']['depth_fields']['name']] = $profondeur;
		}
	}

	// Si besoin on appelle une fonction spécifique au feed pour la liste des enregistrements afin de la finaliser.
	$options = [
		'prefixe' => $feed['feed_id'],
		'include' => $feed['include']
	];
	if ($completer = ezmashup_chercher_service($plugin, 'record_list_completer', $options)) {
		$enregistrements = $completer($enregistrements, $feed);
	}

	return $enregistrements;
}

// -----------------------------------------------------------------------
// ------------------------- SOURCE DATASETS -----------------------------
// -----------------------------------------------------------------------

/**
 * Renvoie la configuration par défaut du dossier relatif où stocker les datasets cible de type fichier.
 *
 * Le service de Mashup Factory considère que par défaut le dossier relatif des types de noisette est `targets/`
 * et est inclus dans le dossier de base _DIR_ETC/ezmashup/.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @uses ezmashup_chercher_service()
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 * @param array  $item   Item d'un dataset source
 * @param array  $feed   Configuration du feed
 *
 * @return array Item mis à jour
 */
function ezmashup_item_completer(string $plugin, array $item, array $feed) : array {
	// Si besoin on appelle une fonction spécifique au feed pour chaque item source afin de le compléter, par exemple
	// - traitement des champs extra_fields
	// - ou modification d'un champ déjà chargé dans l'item.
	$options = [
		'prefixe' => $feed['feed_id'],
		'include' => $feed['include']
	];
	if ($completer = ezmashup_chercher_service($plugin, 'item_completer', $options)) {
		$item = $completer($item, $feed);
	}

	return $item;
}

// -----------------------------------------------------------------------
// ------------------------ SERVICES INTERNES ----------------------------
// -----------------------------------------------------------------------

/**
 * Trouve les fichiers du path correspondants à un pattern et selon les options demandées.
 *
 * Par défaut, pour un nom de fichier donné, ne retourne que le premier qui sera trouvé par un `find_in_path()`.
 * Néanmoins, il est possible de renvoyer tous les fichiers et de formater différemment la sortie dans ce cas.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @internal
 *
 * @uses creer_chemin()
 * @uses preg_files()
 *
 * @param string     $dir     Répertoire de base pour la recherche
 * @param string     $pattern Pattern des fichiers à chercher dans le path
 * @param null|array $options Options permettant de chercher récursivement dans les sous-répertoires (index `recurs`)
 *                            ou de renvoyer tous les fichiers (index `all_files`). Dans ce cas, l'option `flat_output` permet
 *                            de renvoyer une liste des fichiers sans les regrouper par basename.
 *
 * @return array Liste des fichiers trouvés selon le format demandé dans les options
 */
function ezmashup_find_all_in_path(string $dir, string $pattern, ?array $options = []): array {
	$liste_fichiers = [];
	$maxfiles = 10000;

	// cas borderline si dans mes_options on appelle redirige_par_entete qui utilise _T et charge un fichier de langue
	// on a pas encore inclus flock.php
	if (!function_exists('preg_files')) {
		include_once _ROOT_RESTREINT . 'inc/flock.php';
	}

	// Parcourir le chemin
	foreach (creer_chemin() as $d) {
		$f = $d . $dir;
		if (@is_dir($f)) {
			$liste = preg_files($f, $pattern, $maxfiles - count($liste_fichiers), !empty($options['recurs']) ? [] : false);
			foreach ($liste as $chemin) {
				$nom = basename($chemin);
				if (!empty($options['all_files'])) {
					// On accumule tous les fichiers de même nom selon le format demandé
					if (!empty($options['flat_output'])) {
						$liste_fichiers[] = $chemin;
					} else {
						$liste_fichiers[$nom][] = $chemin;
					}
				} elseif (!isset($liste_fichiers[$nom])) {
					// Ne conserver que les fichiers non deja trouves car find_in_path prend le premier qu'il trouve,
					// les autres sont donc masques
					$liste_fichiers[$nom] = $chemin;
				}
			}
		}
	}

	return $liste_fichiers;
}

/**
 * Normalise le chemin de l'include des fonctions spécifiques d'un feed.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @internal
 *
 * @param string      $include      Expression de l'include suivant la syntaxe brute du YAML
 * @param null|string $dossier_feed Dossier relatif des feeds où se trouve le l'include ou vide si l'include est ailleurs
 *
 * @return string Chemin complet de l'include
 */
function ezmashup_normaliser_include(string $include, ?string $dossier_feed = '') {
	// Détermination du chemin exact de l'include
	// - '/nom_fichier' indique que le fichier est dans le répertoire relatif par défaut, soit 'ezmashup/'.
	// - 'nom_fichier' indique que le fichier est dans le répertoire relatif du feed, soit 'dossier_des_feeds/id_du_feed/'
	// - 'dir_relatif/nom_fichier' indique qu'il faut utiliser le chemin relatif fourni tel que.
	$nom_include = basename($include);
	if ($include === "/{$nom_include}") {
		$include = "ezmashup/{$nom_include}";
	} elseif ($include === "//{$nom_include}") {
		$include = "{$dossier_feed}{$nom_include}";
	}

	return $include;
}

/**
 * Compile les traductions du nom, slogan et description d'une boussole, d'un groupe ou d'uns site.
 * La fonction renvoie une balise <multi> pour chaque traduction.
 *
 * @param string $plugin Préfixe du plugin utilisateur.
 * @param string $idiome Identifiant de la boussole
 *
 * @return string Les traductions de l'idiome en multi ou le texte fourni en entrée si celui-ci n'est pas un idiome
 */
function ezmashup_normaliser_idiome(string $plugin, string $idiome) : string {
	// Si le texte n'est pas un idiome on le renvoie tel quel
	$retour = $idiome;

	if (
		$idiome
		and preg_match(_EXTRAIRE_IDIOME, $idiome, $matches)
		and $matches[1]
	) {
		// On détermine le module et le raccourci
		$module = $matches[1];
		$code = $matches[2];

		// Récupérer la liste des langues autorisées
		include_spip('inc/lang_liste');

		// On cherche tous les fichiers de langue destines a la traduction de l'idiome
		$multi = '';
		$dossier = constant('_DIR_PLUGIN_' . strtoupper($plugin));
		if ($fichiers_langue = preg_files($dossier, "/lang/{$module}_[a-z0-9_]+\.php$")) {
			foreach ($fichiers_langue as $_fichier_langue) {
				$nom_fichier = basename($_fichier_langue, '.php');
				$langue = str_replace("{$module}_", '', $nom_fichier);
				// Si la langue est reconnue, on extrait la traduction demandée
				if (isset($GLOBALS['codes_langues'][$langue])) {
					$GLOBALS['idx_lang'] = $langue;
					include("{$_fichier_langue}");
					if (isset($GLOBALS[$langue][$code])) {
						$multi .= "[{$langue}]" . $GLOBALS[$langue][$code];
					}
				}
			}
		}

		if ($multi) {
			$retour = "<multi>{$multi}</multi>";
		}
	}

	return $retour;
}

/**
 * Cherche une fonction donnée en se basant sur le feed ou à défaut sur le plugin appelant.
 * Si ni le feed ni le plugin ne fournissent la fonction demandée la chaîne vide est renvoyée.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @internal
 *
 * @param string     $plugin   Préfixe du plugin utilisateur.
 * @param string     $fonction Nom de la fonction à chercher.
 * @param null|array $options  Options de calcul de la fonction :
 *                             - string include : Include dans lequel chercher la fonction. Si vide, on cherche dans ezmashup/$plugin.
 *                             - string prefixe : préfixe de la fonction. Remplace $plugin comme préfixe
 *                             Ces options sont prioritaires pour la recherche : si la fonction n'est pas trouvée on recherche
 *                             alors la fonction au niveau du plugin
 *
 * @return string Nom complet de la fonction si trouvée ou chaine vide sinon.
 */
function ezmashup_chercher_service(string $plugin, string $fonction, ?array $options = []): string {
	$fonction_trouvee = '';

	// Eviter la réentrance si on demande le plugin Mashup Factory
	if ($plugin !== 'ezmashup') {
		// On inclut toujours le fichier des services personnalisés du plugin utilisateur
		include_spip("ezmashup/{$plugin}");
		if (!empty($options['include'])) {
			include_spip($options['include']);
		}

		// On cherche prioritairement au niveau du feed, puis au niveau du plugin
		$fonctions_a_chercher = [];
		if (!empty($options['prefixe'])) {
			$fonctions_a_chercher[] = "{$options['prefixe']}_{$fonction}";
		}
		$fonctions_a_chercher[] = "{$plugin}_{$fonction}";
		foreach ($fonctions_a_chercher as $_fonction) {
			if (function_exists($_fonction)) {
				$fonction_trouvee = $_fonction;
				break;
			}
		}
	}

	return $fonction_trouvee;
}

/**
 * Construit le message de trace qui sera fourni à la fonction `spip_log`.
 *
 * @package SPIP\MASHUP\SERVICE
 *
 * @internal
 *
 * @param array $erreur Tableau de l'erreur, de l'avertissement ou de l'information à loger :
 *                      - bool   status     : `true` si ok, `false` si erreur.
 *                      - string code       : identifiant de l'erreur qui coincide avec l'idiome du message
 *                      - array  parameters : paramètres pour le calcul du message
 *
 * @return string Message à loger
 */
function ezmashup_log_creer_message(array $erreur): string {
	$texte = '';

	// Construction du message
	if (!empty($erreur['code'])) {
		// Le code d'erreur (non traduit)
		$texte .= strtoupper($erreur['code']);

		// Les paramètres
		include_spip('inc/texte_mini');
		$parametres = '';
		if (!empty($erreur['parameters'])) {
			foreach ($erreur['parameters'] as $_parametre => $_valeur) {
				$parametres .= ($parametres ? ' - ' : '')
					. $_parametre . '=' . couper($_valeur, 200, '...');
			}
		}

		// Le texte complet
		$texte .= ' : ' . rtrim($parametres);
	}

	return $texte;
}
